@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen 
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,44 +15,33 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
 #
-# The developers of the Exaile media player hereby grant permission 
-# for non-GPL compatible GStreamer and Exaile plugins to be used and 
-# distributed together with GStreamer and Exaile. This permission is 
-# above and beyond the permissions granted by the GPL license by which 
-# Exaile is covered. If you modify this code, you may extend this 
-# exception to your version of the code, but you are not obligated to 
-# do so. If you do not wish to do so, delete this exception statement 
+# The developers of the Exaile media player hereby grant permission
+# for non-GPL compatible GStreamer and Exaile plugins to be used and
+# distributed together with GStreamer and Exaile. This permission is
+# above and beyond the permissions granted by the GPL license by which
+# Exaile is covered. If you modify this code, you may extend this
+# exception to your version of the code, but you are not obligated to
+# do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-# Playlist
-#
-# Playlist - essentially an ordered TrackDB
-#
-# SmartPlaylist - playlist that auto-populates from a collection
-#
-# PlaylistManager - playlist persistence manager
-#
-# also contains functions for saving and loading various playlist formats.
-
-from xl import trackdb, event, xdg, track, collection, settings
-import urllib, random, os, time, cgi
-from xl.nls import gettext as _
+import cgi
+from datetime import datetime, timedelta
+import logging
+import os
+import random
+import time
+import urllib
+import urlparse
+import xml.etree.cElementTree as ETree
 
 try:
     import cPickle as pickle
 except:
     import pickle
 
-try:
-    import xml.etree.cElementTree as ETree
-except ImportError:
-    try:
-        import cElementTree as ETree
-    except ImportError:
-        import elementtree as ETree
+from xl.nls import gettext as _
+from xl import event, xdg, collection, settings, trax
 
-import urlparse
-import logging
 logger = logging.getLogger(__name__)
 
 class InvalidPlaylistTypeException(Exception):
@@ -61,7 +50,6 @@ class InvalidPlaylistTypeException(Exception):
 def encode_filename(name):
     """Converts name into a valid filename.
     """
-    import string
     # list of invalid chars that need to be encoded
     # Note: '%' is the prefix for encoded chars so blacklist it too
     blacklist = r'<>:"/\|?*%'
@@ -86,11 +74,17 @@ def save_to_m3u(playlist, path):
         handle.write("#PLAYLIST: %s\n" % playlist.get_name())
 
     for track in playlist:
-        leng = float(track['__length'])
-        if leng < 1: 
+        rawlen = track.get_tag_raw('__length')
+        if rawlen:
+            leng = round(float(rawlen))
+        if not rawlen or leng < 1:
             leng = -1
+        artist = track.get_tag_raw('artist', join=True)
+        title = track.get_tag_raw('title', join=True)
+        if artist:
+            title = artist + ' - ' + title
         handle.write("#EXTINF:%d,%s\n%s\n" % (leng,
-            track['title'], track.get_loc()))
+            title, track.get_loc_for_io()))
 
     handle.close()
 
@@ -105,13 +99,13 @@ def import_from_m3u(path):
         handle = urllib.urlopen(path)
         name = url_parsed[2].split('/')[-1].replace('.m3u', '')
         is_local = False
-    
+
     #if not handle.readline().startswith("#EXTM3U"):
     #    return None
 
     pl = Playlist(name=name)
 
-    current = None
+    current_extinf = None
     for line in handle:
         line = line.strip()
         if line == "":
@@ -119,27 +113,33 @@ def import_from_m3u(path):
         elif line.startswith("#Playlist: "):
             pl.set_name(line[12:])
         elif line.startswith("#EXTINF:"):
-            current = track.Track()
-            comma_separated = line[8:].split(",", 1)
-            title = comma_separated[-1]
-            if len(comma_separated) > 1:
-                length = float(comma_separated[0])
-                if length < 1:
-                    length = 0
-            else:
-                length = 0
-            current['title'] = title
-            current['__length'] = length
+            current_extinf = line
         elif line.startswith("#"):
             pass
         else:
-            if not current:
-                current = track.Track()
             track_is_local = len(urlparse.urlparse(line)[0]) <= 1
             if track_is_local and not os.path.isabs(line):
                 line = os.path.join(os.path.dirname(path), line)
-            current.set_loc(line)
-            current.read_tags()
+            current = trax.Track(line)
+
+            if current_extinf:
+                comma_separated = current_extinf[8:].split(",", 1)
+                title = comma_separated[-1]
+                if len(comma_separated) > 1:
+                    length = float(comma_separated[0])
+                    if length < 1:
+                        length = 0
+                else:
+                    length = 0
+                artist_title = title.split(' - ', 1)
+                if len(artist_title) > 1:
+                    current.set_tag_raw('artist', artist_title[0])
+                    current.set_tag_raw('title', artist_title[1])
+                else:
+                    current.set_tag_raw('title', title)
+                current.set_tag_raw('__length', length)
+                current_extinf = None
+
             pl.add(current, ignore_missing_files=track_is_local)
             current = None
 
@@ -152,21 +152,25 @@ def save_to_pls(playlist, path):
         Saves a Playlist to an pls file
     """
     handle = open(path, "w")
-    
+
     handle.write("[playlist]\n")
     handle.write("NumberOfEntries=%d\n\n" % len(playlist))
-    
-    count = 1 
-    
+
+    count = 1
+
     for track in playlist:
-        handle.write("File%d=%s\n" % (count, track.get_loc()))
-        handle.write("Title%d=%s\n" % (count, track['title']))
-        if track['__length'] < 1:
-            handle.write("Length%d=%d\n\n" % (count, -1))
-        else:
-            handle.write("Length%d=%d\n\n" % (count, float(track['__length'])))
+        artist = track.get_tag_raw('artist', join=True)
+        title = track.get_tag_raw('title', join=True)
+        if artist:
+            title = artist + ' - ' + title
+        handle.write("File%d=%s\n" % (count, track.get_loc_for_io()))
+        handle.write("Title%d=%s\n" % (count, title))
+        length = round(float(track.get_tag_raw('__length') or -1))
+        if length < 0:
+            length = -1
+        handle.write("Length%d=%d\n\n" % (count, length))
         count += 1
-    
+
     handle.write("Version=2")
     handle.close()
 
@@ -193,7 +197,11 @@ def import_from_pls(path, handle=None):
             return None
 
     if not linedict.has_key("version"):
-        return None
+        logger.warning("No PLS version specified, assuming 2. [%s]"%path)
+    else:
+        if linedict["version"].strip() != '2':
+            logger.error("PLS file is not a supported version!")
+            return None
     if not linedict.has_key("numberofentries"):
         return None
 
@@ -202,50 +210,53 @@ def import_from_pls(path, handle=None):
     pl = Playlist(name=name)
 
     for n in range(1,num+1):
-        tr = track.Track()
-        tr.set_loc(linedict["file%s"%n])
-        if "title%s"%n in linedict:
-            tr['title'] = linedict["title%s"%n]
+        tr = trax.Track(linedict["file%d" % n])
+        if ("title%d" % n) in linedict:
+            title = linedict["title%d" % n]
+            artist_title = title.split(' - ', 1)
+            if len(artist_title) > 1:
+                artist, title = artist_title
+                tr.set_tag_raw('artist', artist)
         else:
-            tr['title'] = linedict["file%s"%n].split("/")[-1]
-        if "__length%s"%n in linedict:
-            len = float(linedict["__length%s"%n])
+            title = os.path.splitext(os.path.basename(linedict["file%d" % n]))[0]
+        tr.set_tag_raw('title', title)
+        if ("Length%d" % n) in linedict:
+            length = float(linedict["Length%d" % n])
+            if length < 0:
+                length = 0
         else:
-            len = 0
-        if len < 1:
-            len = 0
-        tr['__length'] = len
-        if tr.get_type() == 'file': # FIXME
-            tr.read_tags()
+            length = 0
+        tr.set_tag_raw('__length', length)
         pl.add(tr, ignore_missing_files=False)
 
     handle.close()
 
     return pl
 
-    
+
 def save_to_asx(playlist, path):
     """
         Saves a Playlist to an asx file
     """
     handle = open(path, "w")
-    
+
     handle.write("<asx version=\"3.0\">\n")
     if playlist.get_name() == '':
         name = ''
     else:
         name = playlist.get_name()
     handle.write("  <title>%s</title>\n" % name)
-    
+
     for track in playlist:
         handle.write("<entry>\n")
-        handle.write("  <title>%s</title>\n" % track['title'])
-        handle.write("  <ref href=\"%s\" />\n" % track.get_loc())
+        handle.write("  <title>%s</title>\n" % \
+                track.get_tag_raw('title', join=True))
+        handle.write("  <ref href=\"%s\" />\n" % track.get_loc_for_io())
         handle.write("</entry>\n")
-    
+
     handle.write("</asx>")
     handle.close()
-    
+
 def import_from_asx(path):
     tree = ETree.ElementTree(file=urllib.urlopen(path))
     # bad hack to support non-lowercase elems. FIXME
@@ -264,14 +275,12 @@ def import_from_asx(path):
             break
     pl = Playlist(name=name)
     for t in tracks:
-        tr = track.Track()
         loc = t.find("ref").get("href")
-        tr.set_loc(loc)
+        tr = trax.Track(loc)
         try:
-            tr['title'] = t.find("title").text.strip()
+            tr.set_tag_raw('title', t.find("title").text.strip())
         except:
             pass
-        tr.read_tags()
         pl.add(tr, ignore_missing_files=False)
     return pl
 
@@ -287,44 +296,53 @@ def save_to_xspf(playlist, path):
         Saves a Playlist to a xspf file
     """
     handle = open(path, "w")
-    
+
     handle.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
     handle.write("<playlist version=\"1\" xmlns=\"http://xspf.org/ns/0/\">\n")
     if playlist.get_name() != '':
         handle.write("  <title>%s</title>\n" % playlist.get_name())
-    
+
     handle.write("  <trackList>\n")
     for track in playlist:
         handle.write("    <track>\n")
         for xs, tag in XSPF_MAPPING.iteritems():
-            if track[tag] == u"":
+            if not track.get_tag_raw(tag):
                 continue
-            handle.write("      <%s>%s</%s>\n" % (xs, track[tag],xs) )
-        url = track.get_loc()
+            handle.write("      <%s>%s</%s>\n" % (xs,
+                track.get_tag_raw(tag, join=True),xs) )
+        url = track.get_loc_for_io()
         handle.write("      <location>%s</location>\n" % url)
         handle.write("    </track>\n")
-    
+
     handle.write("  </trackList>\n")
     handle.write("</playlist>\n")
     handle.close()
-    
+
 def import_from_xspf(path):
     #TODO: support content resolution
     tree = ETree.ElementTree(file=urllib.urlopen(path))
     ns = "{http://xspf.org/ns/0/}"
     tracks = tree.find("%strackList"%ns).findall("%strack"%ns)
-    name = tree.find("%stitle"%ns).text.strip()
+    titlenode = tree.find("%stitle"%ns)
+
+    if titlenode is not None:
+        name = titlenode.text.strip()
+    else:
+        import gio
+        gfile = gio.File(path)
+        name = gfile.get_basename()
+        name = name[:-len('.xspf')]
+
     pl = Playlist(name=name)
+
     for t in tracks:
-        tr = track.Track()
         loc = t.find("%slocation"%ns).text.strip()
-        tr.set_loc(loc)
+        tr = trax.Track(loc)
         for xs, tag in XSPF_MAPPING.iteritems():
             try:
-                tr[tag] = t.find("%s%s"%(ns,xs)).text.strip()
+                tr.set_tag_raw(tag, t.find("%s%s"%(ns,xs)).text.strip())
             except:
                 pass
-        tr.read_tags()
         pl.add(tr)
     return pl
 
@@ -356,10 +374,10 @@ def import_playlist(path):
         return import_from_xspf(path)
     else:
         raise InvalidPlaylistTypeException()
-    
+
 def export_playlist(playlist, path):
     """
-        Exact same as @see import_playlist except 
+        Exact same as @see import_playlist except
         it exports
     """
     sections = path.split('.')
@@ -374,7 +392,7 @@ def export_playlist(playlist, path):
         return save_to_xspf(playlist, path)
     else:
         raise InvalidPlaylistTypeException()
-    
+
 
 class PlaylistIterator(object):
     def __init__(self, pl):
@@ -391,8 +409,7 @@ class PlaylistIterator(object):
 
 class Playlist(object):
     """
-        Represents a playlist, which is basically just a TrackDB
-        with ordering.
+        Represents a playlist
     """
     def __init__(self, name=_("Playlist %d"), is_custom=False):
         """
@@ -410,27 +427,30 @@ class Playlist(object):
         self.current_pos = -1
         self.current_playing = False
         self.random_enabled = False
+        self.random_mode = "track"
         self.repeat_enabled = False
+        self.repeat_mode = "playlist"
         self.dynamic_enabled = False
         self._is_custom = is_custom
         self._needs_save = False
         self.name = name
         self.tracks_history = []
-        self.extra_save_items = ['random_enabled', 'repeat_enabled', 
-                'dynamic_enabled', 'current_pos', 'name', '_is_custom', '_needs_save']
+        self.extra_save_items = ['random_enabled', 'random_mode',
+            'repeat_enabled', 'repeat_mode', 'dynamic_enabled',
+            'current_pos', 'name', '_is_custom', '_needs_save']
 
     def get_name(self):
         return self.name
 
     def set_name(self, name):
         self.name = name
-        
+
     def get_is_custom(self):
         return self._is_custom
 
     def set_is_custom(self, val):
         self._is_custom = val
-        
+
     def get_needs_save(self):
         return self._needs_save
 
@@ -443,7 +463,7 @@ class Playlist(object):
         """
         if self.filtered:
             return self.filtered_tracks
-        else:   
+        else:
             return self._ordered_tracks
 
     def _set_ordered_tracks(self, tracks):
@@ -474,8 +494,7 @@ class Playlist(object):
             self.filtered = False
             return self._ordered_tracks
         else:
-            self.filtered_tracks = self.search(keyword,
-                sort_fields=('artist', 'date', 'album', 'discnumber', 'tracknumber'))
+            self.filtered_tracks = list(self.search(keyword))
             self.filtered = True
             return self.filtered_tracks
 
@@ -494,7 +513,16 @@ class Playlist(object):
         """
         return PlaylistIterator(self)
 
-    def add(self, track, location=None, ignore_missing_files = True):
+    def __contains__(self, track):
+        return track in self._ordered_tracks
+
+    def __getitem__(self, item):
+        return self.ordered_tracks.__getitem__(item)
+
+    def __getslice__(self, one, two):
+        return self.ordered_tracks.__getslice__(one, two)
+
+    def add(self, track, location=None, ignore_missing_files=True):
         """
             insert the track into the playlist at the specified
             location (default: append). by default it the track can
@@ -508,28 +536,23 @@ class Playlist(object):
         if track.exists() or not ignore_missing_files:
             self.add_tracks([track], location)
 
-    def add_tracks(self, tracks, location=None, add_duplicates=True):
+    def add_tracks(self, tracks, location=None):
         """
             like add(), but takes a list of tracks instead of a single one
 
-            @param tracks: the tracks to add [list of Track]
+            @param tracks: the tracks to add [iterable of Track]
             @param location: the index to insert at [int]
             @param add_duplicates: Set to [False] if you wouldn't like to add
                 tracks that are already in the playlist
         """
-        if not add_duplicates:
-            new = []
-            for track in tracks:
-                if track not in self.get_tracks():
-                    new.append(track)
-            tracks = new
-
         if location == None:
             self.ordered_tracks.extend(tracks)
         else:
-            self.ordered_tracks = self.ordered_tracks[:location] + \
-                    tracks + self.ordered_tracks[location:]
-        
+            neworder = self.ordered_tracks[:location]
+            neworder.extend(tracks)
+            neworder.extend(self.ordered_tracks[location:])
+            self.ordered_tracks = neworder
+
         if location != None and location <= self.current_pos:
             self.current_pos += len(tracks)
 
@@ -561,9 +584,9 @@ class Playlist(object):
         self.ordered_tracks = self.ordered_tracks[:start] + \
                 self.ordered_tracks[end:]
 
-        if end < self.current_pos:
+        if end <= self.current_pos:
             self.current_pos -= len(removed)
-        elif start <= self.current_pos <= end:
+        elif start <= self.current_pos < end:
             self.current_pos = start-1
 
         event.log_event('tracks_removed', self, (start, end, removed))
@@ -619,51 +642,96 @@ class Playlist(object):
 
     def peek(self):
         """
-            returns the next track that will be played
+            Returns the next track that will be played
         """
         if self.random_enabled:
             return None #peek() is meaningless with random
-        nextpos = self.current_pos + 1
+
+        if self.repeat_enabled and self.repeat_mode == 'track':
+            nextpos = self.current_pos
+        else:
+            nextpos = self.current_pos + 1
+
         if nextpos >= len(self):
-            if self.repeat_enabled:
+            if self.repeat_enabled and self.repeat_mode == 'playlist':
                 nextpos = 0
             else:
                 return None # end of playlist
         return self.ordered_tracks[nextpos]
 
+    def get_next_random_track(self, mode="track"):
+        """
+            Returns a valid next track if shuffle is activated based
+            on random_mode
+        """
+        if mode == "album":
+            try:
+                # Try and get the next track on the album
+                # NB If the user starts the playlist from the middle
+                # of the album some tracks of the album remain off the
+                # tracks_history, and the album can be selected again
+                # randomly from its first track
+                curr = self.ordered_tracks[self.current_pos]
+                t = [ x for i, x in enumerate(self.ordered_tracks) \
+                    if x.get_tag_raw('album') == curr.get_tag_raw('album') \
+                    and i > self.current_pos ]
+                t = trax.sort_tracks(['discnumber', 'tracknumber'], t)
+                return t[0]
+
+            except IndexError: #Pick a new album
+                t = [ x for x in self.ordered_tracks \
+                        if x not in self.tracks_history ]
+                albums = []
+                for x in t:
+                    if not x.get_tag_raw('album') in albums:
+                        albums.append(x.get_tag_raw('album'))
+
+                album = random.choice(albums)
+                t = [ x for x in self.ordered_tracks \
+                        if x.get_tag_raw('album') == album ]
+                t = trax.sort_tracks(['tracknumber'], t)
+                return t[0]
+        else:   # track mode - dont check explicitly because the restore code
+                # sometimes gives us a None here.
+            return random.choice([ x for x in self.ordered_tracks \
+                    if x not in self.tracks_history])
+
+
     def next(self):
         """
             moves to the next track in the playlist
 
             returns: the next track [Track], None if no more tracks
         """
-        if self.random_enabled:
+        if self.repeat_enabled and self.repeat_mode == 'track':
+            pass # Stay where we are
+        elif self.random_enabled:
             if self.current_pos != -1:
                 self.tracks_history.append(self.get_current())
-                if self.repeat_enabled and len(self.tracks_history) >= \
-                        len(self.ordered_tracks):
+                if self.repeat_enabled and \
+                    len(self.tracks_history) >= len(self.ordered_tracks):
                     self.tracks_history = []
             if len(self.ordered_tracks) == 1 and \
                     len(self.tracks_history) == 1:
                 return None
-            
+
             try:
-                next = random.choice([ x for x in self.ordered_tracks \
-                    if x not in self.tracks_history])
+                next = self.get_next_random_track(self.random_mode)
+
             except IndexError:
                 logger.debug('Ran out of tracks to shuffle')
-                # clear this so we restart the shuffle cycle next time a 
+                # clear this so we restart the shuffle cycle next time a
                 # track is played
                 self.tracks_history = []
                 return None
-            self.current_pos = self.ordered_tracks.index(next)            
+            self.current_pos = self.ordered_tracks.index(next)
         else:
             if len(self.ordered_tracks) == 0:
                 return None
             self.current_pos += 1
 
         if self.current_pos >= len(self.ordered_tracks):
-            if self.repeat_enabled:
+            if self.repeat_enabled and self.repeat_mode == 'playlist':
                 self.current_pos = 0
             else:
                 self.current_pos = -1
@@ -689,7 +757,7 @@ class Playlist(object):
         else:
             self.current_pos -= 1
             if self.current_pos < 0:
-                if self.repeat_enabled:
+                if self.repeat_enabled and self.repeat_mode == 'playlist':
                     self.current_pos = len(self.ordered_tracks) - 1
                 else:
                     self.current_pos = 0
@@ -703,23 +771,21 @@ class Playlist(object):
         """
             searches the playlist
         """
-        searcher = trackdb.TrackSearcher()
-
-        search_db = {}
-        for track in self.ordered_tracks:
-            search_db[track.get_loc()] = track
-        tracks = searcher.search(phrase, search_db)
-        tracks = tracks.values()
+        # TODO: use shown columns
+        matcher = trax.TracksMatcher(phrase, keyword_tags=('artist',
+            'album', 'title'), case_sensitive=False)
+        trs = trax.search_tracks(self._ordered_tracks, [matcher])
+        trs = (t.track for t in trs)
 
         if sort_fields:
             if sort_fields == 'RANDOM':
-                random.shuffle(tracks)
+                random.shuffle(trs)
             else:
-                tracks = trackdb.sort_tracks(sort_fields, tracks)
+                trs = trax.sort_tracks(sort_fields, trs)
         if return_lim != -1:
-            tracks = tracks[:return_lim]
+            trs = trs[:return_lim]
 
-        return tracks
+        return trs
 
     def toggle_random(self):
         """
@@ -727,41 +793,46 @@ class Playlist(object):
         """
         if not self.random_enabled:
             self.tracks_history = []
-        self.random_enabled = self.random_enabled == False
+        self.random_enabled = not self.random_enabled
         self._dirty = True
 
     def toggle_repeat(self):
         """
             toggle repeat playback
         """
-        self.repeat_enabled = self.repeat_enabled == False
+        self.repeat_enabled = not self.repeat_enabled
         self._dirty = True
 
     def toggle_dynamic(self):
         """
             toggle dynamic adding of similar tracks to the playlist
         """
-        self.dynamic_enabled = self.repeat_enabled == False
+        self.dynamic_enabled = not self.repeat_enabled
         self._dirty = True
 
-    def set_random(self, value):
+    def set_random(self, value, mode="track"):
         """
             Enables random mode if it isn't already enabled
 
-            @param value: [bool]
+            :param value: [bool]
         """
         if not self.random_enabled:
             self.tracks_history = []
         self.random_enabled = value
+        if mode != self.random_mode:
+            #Makes shuffle a bit more interesting if switching random mode
+            self.tracks_history = []
+            self.random_mode = mode
         self._dirty = True
 
-    def set_repeat(self, value):
+    def set_repeat(self, value, mode="playlist"):
         """
             Enables repeat mode if it isn't already enabled
 
-            @param value: [bool]
+            :param value: [bool]
         """
         self.repeat_enabled = value
+        self.repeat_mode = mode
         self._dirty = True
 
     def set_dynamic(self, value):
@@ -793,22 +864,30 @@ class Playlist(object):
             f = open(location + ".new", "w")
         else:
             f = open(location, "w")
-        for tr in self:
-            buffer = tr.get_loc()
+        for tr in self._ordered_tracks:
+            buffer = tr.get_loc_for_io()
             # write track metadata
             meta = {}
             items = ('artist', 'album', 'tracknumber', 'title', 'genre',
                 'date')
             for item in items:
-                value = tr[item]
-                if value is not None: meta[item] = value[0]
+                value = tr.get_tag_raw(item)
+                if value is not None:
+                    meta[item] = value[0]
             buffer += '\t%s\n' % urllib.urlencode(meta)
-            f.write(buffer.encode('utf-8'))
+            try:
+                f.write(buffer.encode('utf-8'))
+            except UnicodeDecodeError:
+                continue
 
         f.write("EOF\n")
         for item in self.extra_save_items:
             val = getattr(self, item)
-            strn = settings._SETTINGSMANAGER._val_to_str(val)
+            try:
+                strn = settings.MANAGER._val_to_str(val)
+            except ValueError:
+                strn = ""
+
             f.write("%s=%s\n"%(item,strn))
         f.close()
         if os.path.exists(location + ".new"):
@@ -829,43 +908,39 @@ class Playlist(object):
             line = f.readline()
             if line == "EOF\n" or line == "":
                 break
-            locs.append(line.decode('utf-8').strip())
+            locs.append(line.strip())
         while True:
             line = f.readline()
             if line == "":
                 break
             item, strn = line[:-1].split("=",1)
-            val = settings._SETTINGSMANAGER._str_to_val(strn)
+            val = settings.MANAGER._str_to_val(strn)
             if hasattr(self, item):
                 setattr(self, item, val)
         f.close()
 
-        tracks = []
+        trs = []
 
         for loc in locs:
             meta = None
             if loc.find('\t') > -1:
-                (loc, meta) = loc.split('\t')
-               
+                splitted = loc.split('\t')
+                loc = "\t".join(splitted[:-1])
+                meta = splitted[-1]
+
             tr = None
-            col = collection.get_collection_by_loc(loc)
-            if col:
-                tr = col.get_track_by_loc(loc)
-            if not tr:
-                tr = track.Track(uri=loc)
-                if tr.is_local() and not tr._scan_valid:
-                    tr = None
+            tr = trax.Track(uri=loc)
 
             # readd meta
             if not tr: continue
             if not tr.is_local() and meta is not None:
                 meta = cgi.parse_qs(meta)
                 for k, v in meta.iteritems():
-                    tr[k] = v[0]
+                    tr.set_tag_raw(k, v[0], notify_changed=False)
 
-            tracks.append(tr)
+            trs.append(tr)
 
-        self.ordered_tracks = tracks
+        self.ordered_tracks = trs
 
     def randomize(self):
         """
@@ -874,11 +949,11 @@ class Playlist(object):
         trs = self.ordered_tracks
         random.shuffle(trs)
         self.ordered_tracks = trs
-    
+
 
 class SmartPlaylist(object):
-    """ 
-        Represents a Smart Playlist.  
+    """
+        Represents a Smart Playlist.
         This will query a collection object using a set of parameters
 
         Simple usage:
@@ -892,14 +967,13 @@ class SmartPlaylist(object):
         >>> p = sp.get_playlist()
         >>> p.get_tracks()[1]['album'][0]
         u'Chimera'
-        >>> 
+        >>>
     """
     def __init__(self, name="", collection=None):
         """
             Sets up a smart playlist
 
             @param collection: a reference to a TrackDB object.
-            args: See TrackDB
         """
         self.search_params = []
         self.custom_params = []
@@ -927,7 +1001,7 @@ class SmartPlaylist(object):
         self.collection = collection
 
     def set_random_sort(self, sort):
-        """ 
+        """
             If True, the tracks added during update() will be randomized
 
             @param sort: bool
@@ -940,10 +1014,10 @@ class SmartPlaylist(object):
             Returns True if this playlist will randomly be sorted
         """
         return self.random_sort
-    
+
     def set_return_limit(self, count):
         """
-            Sets the max number of tracks to return.  
+            Sets the max number of tracks to return.
 
             @param count:  number of tracks to return.  Set to -1 to return
                 all matched
@@ -959,7 +1033,8 @@ class SmartPlaylist(object):
 
     def set_or_match(self, value):
         """
-            Set to True to make this an or match: match any of the parameters
+            Set to True to make this an or match: match any of the
+            parameters
 
             value: True to match any, False to match all params
         """
@@ -971,7 +1046,7 @@ class SmartPlaylist(object):
             Return if this is an any or and playlist
         """
         return self.or_match
-        
+
     def add_param(self, field, op, value, index=-1):
         """
             Adds a search parameter.
@@ -980,8 +1055,8 @@ class SmartPlaylist(object):
             @param op:     The operator.  Valid operators are:
                     >,<,>=,<=,=,!=,==,!==,>< (between) [string]
             @param value:  The value to match against [string]
-            @param index:  Where to insert the parameter in the search order.  -1 
-                    to append [int]
+            @param index:  Where to insert the parameter in the search
+                    order.  -1 to append [int]
         """
         if index:
             self.search_params.insert(index, [field, op, value])
@@ -1006,7 +1081,7 @@ class SmartPlaylist(object):
     def remove_param(self, index):
         """
             Removes a parameter at the speficied index
-        
+
             index:  the index of the parameter to remove
         """
         self._dirty = True
@@ -1015,9 +1090,9 @@ class SmartPlaylist(object):
     def get_playlist(self, collection=None):
         """
             Generates a playlist by querying the collection
-            
-            @param collection: the collection to search (leave None to search
-                        internal ref)
+
+            @param collection: the collection to search (leave None to
+                    search internal ref)
         """
         if not collection:
             collection = self.collection
@@ -1025,18 +1100,21 @@ class SmartPlaylist(object):
             return
 
         search_string = self._create_search_string()
-        sort_field = None
-        if self.random_sort: 
-            sort_field = 'RANDOM'
+
+        matcher = trax.TracksMatcher(search_string, case_sensitive=False)
+        trs = [ t.track for t in trax.search_tracks(collection, [matcher]) ]
+        if self.random_sort:
+            random.shuffle(trs)
         else:
-            sort_field = ('artist', 'date', 'album', 'discnumber', 'tracknumber', 'title')
+            sort_field = ('artist', 'date', 'album', 'discnumber',
+                    'tracknumber', 'title')
+            trs = trax.sort_tracks(sort_field, trs)
+        if self.track_count > 0 and len(trs) > self.track_count:
+            trs=trs[:self.track_count]
+
 
         pl = Playlist(name=self.get_name())
-
-        tracks = list(collection.search(search_string, sort_field,
-            self.track_count))
-
-        pl.add_tracks(tracks)
+        pl.add_tracks(trs)
 
         return pl
 
@@ -1046,7 +1124,14 @@ class SmartPlaylist(object):
         """
 
         params = [] # parameter list
-        steps = settings.get_option('miscellaneous/rating_steps', 5)
+        maximum = settings.get_option('rating/maximum', 5)
+        durations = {
+            _('seconds'): lambda value: timedelta(seconds=value),
+            _('minutes'): lambda value: timedelta(minutes=value),
+            _('hours'): lambda value: timedelta(hours=value),
+            _('days'): lambda value: timedelta(days=value),
+            _('weeks'): lambda value: timedelta(weeks=value),
+        }
 
         for param in self.search_params:
             if type(param) == str:
@@ -1056,24 +1141,30 @@ class SmartPlaylist(object):
             s = ""
 
             if field == '__rating':
-                value = float((100.0*value)/steps)
+                value = float((100.0*value)/maximum)
+            elif field in ('__date_added', '__last_played'):
+                duration, unit = value
+                delta = durations[unit](duration)
+                point = datetime.now() - delta
+                value = time.mktime(point.timetuple())
+
             if op == ">=" or op == "<=":
                 s += '( %(field)s%(op)s%(value)s ' \
-                    'OR %(field)s==%(value)s )' % \
+                    '| %(field)s==%(value)s )' % \
                     {
                         'field': field,
                         'value': value,
                         'op':    op[0]
                     }
             elif op == "!=" or op == "!==":
-                s += 'NOT %(field)s%(op)s"%(value)s"' % \
+                s += '! %(field)s%(op)s"%(value)s"' % \
                     {
                         'field': field,
                         'value': value,
                         'op':    op[1:]
                     }
             elif op == "><":
-                s+= '( %(field)s>%(value1)s AND ' \
+                s+= '( %(field)s>%(value1)s ' \
                     '%(field)s<%(value2)s )' % \
                     {
                         'field':  field,
@@ -1091,7 +1182,7 @@ class SmartPlaylist(object):
             params.append(s)
 
         if self.or_match:
-            return ' OR '.join(params)
+            return ' | '.join(params)
         else:
             return ' '.join(params)
 
@@ -1151,9 +1242,10 @@ class PlaylistManager(object):
             pl.save_to_location(os.path.join(self.playlist_dir,
                 encode_filename(name)))
 
-            if not name in self.playlists: 
+            if not name in self.playlists:
                 self.playlists.append(name)
             #self.playlists.sort()
+            self.save_order()
         else:
             raise PlaylistExists
 
@@ -1174,7 +1266,7 @@ class PlaylistManager(object):
                 pass
             self.playlists.remove(name)
             event.log_event('playlist_removed', self, name)
-            
+
     def rename_playlist(self, playlist, new_name):
         """
             Renames the playlist to new_name
@@ -1203,7 +1295,7 @@ class PlaylistManager(object):
             ordered_playlists = self.load_from_location(self.order_file)
             self.playlists = [n for n in ordered_playlists if n in existing]
         else:
-            self.playlists = existing 
+            self.playlists = existing
 
     def get_playlist(self, name):
         """
@@ -1224,7 +1316,7 @@ class PlaylistManager(object):
             Returns all the contained playlist names
         """
         return self.playlists[:]
-        
+
     def move(self, playlist, position, after = True):
         """
             Moves the playlist to where position is
@@ -1237,13 +1329,13 @@ class PlaylistManager(object):
         if after:
             position_index = position_index + 1
         self.playlists.insert(position_index, playlist)
-    
+
     def save_order(self):
         """
             Saves the order to the order file
         """
         self.save_to_location(self.order_file)
-    
+
     def save_to_location(self, location):
         """
             Saves the names of the playlist to a file that is
@@ -1267,7 +1359,7 @@ class PlaylistManager(object):
         """
             Loads the names of the playlist from a file.
             Their load order is their view order
-            
+
             @return: a list of the playlist names
         """
         f = None
