@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,21 +24,25 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-import os.path
 import threading
-import time
-import urllib
 from urllib2 import urlparse
 
 import gio
-import gobject
+import glib
 import gtk
-import gtk.gdk
-import pango
 
-from xl import xdg, playlist, common, settings, event, trax
+from xl import (
+    common,
+    covers,
+    event,
+    formatter,
+    playlist,
+    settings,
+    trax,
+    xdg,
+)
 from xl.nls import gettext as _
-from xlgui import rating
+from xlgui import icons
 
 def _idle_callback(func, callback, *args, **kwargs):
     value = func(*args, **kwargs)
@@ -47,7 +51,7 @@ def _idle_callback(func, callback, *args, **kwargs):
 
 def idle_add(callback=None):
     """
-        A decorator that will wrap the function in a gobject.idle_add call
+        A decorator that will wrap the function in a glib.idle_add call
 
         NOTE: Although this decorator will probably work in more cases than
         the gtkrun decorator does, you CANNOT expect to get a return value
@@ -60,7 +64,7 @@ def idle_add(callback=None):
     """
     def wrap(f):
         def wrapped(*args, **kwargs):
-            gobject.idle_add(_idle_callback, f, callback,
+            glib.idle_add(_idle_callback, f, callback,
                 *args, **kwargs)
 
         return wrapped
@@ -102,6 +106,40 @@ def get_workarea_size():
 
     return rootwindow.property_get(workarea)[2][2:4] # W,H
 
+def gtk_widget_replace(widget, replacement):
+    """
+        Replaces one widget with another and
+        places it exactly at the original position
+
+        :param widget: The original widget
+        :type widget: :class:`gtk.Widget`
+        :param replacement: The new widget
+        :type widget: :class:`gtk.Widget`
+    """
+    parent = widget.get_parent()
+
+    try:
+        position = parent.get_children().index(widget)
+    except AttributeError: # None, not gtk.Container
+        return
+    else:
+        try:
+            packing = parent.query_child_packing(widget)
+        except: # Not gtk.Box
+            pass
+
+        parent.remove(widget)
+        replacement.unparent()
+        parent.add(replacement)
+
+        try:
+            parent.set_child_packing(replacement, *packing)
+            parent.reorder_child(replacement, position)
+        except AttributeError: # Not gtk.Box
+            pass
+
+        replacement.show_all()
+
 class ScalableImageWidget(gtk.Image):
     """
         Custom resizeable image widget
@@ -129,10 +167,10 @@ class ScalableImageWidget(gtk.Image):
         self._set_image(self.pixbuf, fill)
 
     def set_image_data(self, data, fill=False):
-        loader = gtk.gdk.PixbufLoader()
-        loader.write(data)
-        loader.close()
-        self.pixbuf = loader.get_pixbuf()
+        if not data:
+            return
+
+        self.pixbuf = icons.MANAGER.pixbuf_from_data(data)
         self._set_image(self.pixbuf, fill)
 
     def _set_image(self, pixbuf, fill=False):
@@ -156,13 +194,18 @@ class DragTreeView(gtk.TreeView):
     """
     targets = [("text/uri-list", 0, 0)]
     dragged_data = dict()
-    
-    def __init__(self, cont, receive=True, source=True):
+
+    def __init__(self, container, receive=True, source=True, drop_pos=None):
         """
             Initializes the tree and sets up the various callbacks
+            :param container: The container to place the TreeView into
+            :param receive: True if the TreeView should receive drag events
+            :param source: True if the TreeView should send drag events
+            :param drop_pos: Indicates where a drop operation should occur
+                    w.r.t. existing entries: 'into', 'between', or None (both).
         """
         gtk.TreeView.__init__(self)
-        self.cont = cont
+        self.container = container
 
         if source:
             self.drag_source_set(
@@ -170,45 +213,60 @@ class DragTreeView(gtk.TreeView):
                 gtk.gdk.ACTION_COPY|gtk.gdk.ACTION_MOVE)
 
         if receive:
+            self.drop_pos = drop_pos
             self.drag_dest_set(gtk.DEST_DEFAULT_ALL, self.targets,
                 gtk.gdk.ACTION_COPY|gtk.gdk.ACTION_DEFAULT|
                 gtk.gdk.ACTION_MOVE)
             self.connect('drag_data_received',
-                self.cont.drag_data_received)
+                self.container.drag_data_received)
             self.connect('drag_data_delete',
-                self.cont.drag_data_delete)
+                self.container.drag_data_delete)
         self.receive = receive
         self.dragging = False
-        self.connect('drag_begin', self.drag_begin)
-        self.connect('drag_end', self.drag_end)
-        self.connect('drag_motion', self.drag_motion)
-        self.connect('button_release_event', self.button_release)
-        self.connect('button_press_event', self.button_press)
+        self.show_cover_drag_icon = True
+        self.connect('drag-begin', self.on_drag_begin)
+        self.connect('drag-end', self.on_drag_end)
+        self.connect('drag-motion', self.on_drag_motion)
+        self.connect('button-release-event', self.on_button_release)
+        self.connect('button-press-event', self.on_button_press)
 
         if source:
-            self.connect('drag_data_get', self.cont.drag_get_data)
-            self.drag_source_set_icon_stock('gtk-dnd')
+            self.connect('drag-data-get', self.container.drag_get_data)
+            self.drag_source_set_icon_stock(gtk.STOCK_DND)
 
-    def button_release(self, button, event):
+    def get_selected_tracks(self):
+        """
+            Returns the currently selected tracks (stub)
+        """
+        pass
+
+    def on_button_release(self, button, event):
         """
             Called when a button is released
         """
         if event.button != 1 or self.dragging:
             self.dragging = False
             return True
+
         if event.state & (gtk.gdk.SHIFT_MASK|gtk.gdk.CONTROL_MASK):
             return True
+
         selection = self.get_selection()
         selection.unselect_all()
 
-        x, y = event.get_coords()
-        x = int(x); y = int(y)
+        path = self.get_path_at_pos(int(event.x), int(event.y))
+
+        if not path:
+            return False
 
-        path = self.get_path_at_pos(x, y)
-        if not path: return False
         selection.select_path(path[0])
 
-    def drag_end(self, list, context):
+        try:
+            return self.container.button_release(button, event)
+        except AttributeError:
+            pass
+
+    def on_drag_end(self, list, context):
         """
             Called when the dnd is ended
         """
@@ -217,32 +275,128 @@ class DragTreeView(gtk.TreeView):
         self.drag_dest_set(gtk.DEST_DEFAULT_ALL, self.targets,
             gtk.gdk.ACTION_COPY|gtk.gdk.ACTION_MOVE)
 
-    def drag_begin(self, list, context):
+    def on_drag_begin(self, widget, context):
         """
-            Called when dnd is started
+            Sets the cover of dragged tracks as drag icon
         """
         self.dragging = True
-
         context.drag_abort(gtk.get_current_event_time())
-        selection = self.get_selection()
-        if selection.count_selected_rows() > 1:
-            self.drag_source_set_icon_stock('gtk-dnd-multiple')
-        else:
-            self.drag_source_set_icon_stock('gtk-dnd')
 
-    def drag_motion(self, treeview, context, x, y, timestamp):
+        self._on_drag_begin(widget, context)
+
+    @common.threaded
+    def _on_drag_begin(self, widget, context):
+        """
+            Async call counterpart to on_drag_begin, so that cover fetching
+            doesn't block dragging.
+        """
+        if self.show_cover_drag_icon:
+            tracks = self.get_selected_tracks()
+            cover_manager = covers.MANAGER
+            width = height = settings.get_option('gui/cover_width', 100)
+
+            if tracks:
+                tracks = trax.util.sort_tracks(['album', 'tracknumber'], tracks)
+                pixbuf = None
+                first_pixbuf = None
+                albums = []
+
+                for track in tracks:
+                    album = track.get_tag_raw('album', join=True)
+                    if album not in albums:
+                        image_data = cover_manager.get_cover(track)
+                        if image_data is not None:
+                            pixbuf = icons.MANAGER.pixbuf_from_data(
+                                image_data, (width, height))
+
+                            if first_pixbuf is None:
+                                first_pixbuf = pixbuf
+                            albums += [album]
+
+                            if len(albums) >= 2:
+                                break
+
+                if pixbuf is not None:
+                    cover_pixbuf = pixbuf
+
+                    if len(albums) > 1:
+                        # Create stacked-cover effect
+                        cover_pixbuf = gtk.gdk.Pixbuf(
+                            gtk.gdk.COLORSPACE_RGB,
+                            True,
+                            8,
+                            width + 10, height + 10
+                        )
+
+                        fill_pixbuf = cover_pixbuf.subpixbuf(
+                            0, 0, width + 10, height + 10)
+                        fill_pixbuf.fill(0x00000000) # Fill with transparent background
+
+                        fill_pixbuf = cover_pixbuf.subpixbuf(
+                            0, 0, width, height)
+                        fill_pixbuf.fill(0xccccccff)
+
+                        if first_pixbuf != pixbuf:
+                            pixbuf.copy_area(
+                                0, 0, width, height,
+                                cover_pixbuf,
+                                5, 5
+                            )
+                        else:
+                            fill_pixbuf = cover_pixbuf.subpixbuf(
+                                5, 5, width, height)
+                            fill_pixbuf.fill(0x999999ff)
+
+                        first_pixbuf.copy_area(
+                            0, 0, width, height,
+                            cover_pixbuf,
+                            10, 10
+                        )
+
+                    glib.idle_add(self._set_drag_cover, context, cover_pixbuf)
+        else:
+            if self.get_selection().count_selected_rows() > 1:
+                self.drag_source_set_icon_stock(gtk.STOCK_DND_MULTIPLE)
+            else:
+                self.drag_source_set_icon_stock(gtk.STOCK_DND)
+
+    def _set_drag_cover(self, context, pixbuf):
+        """
+            Completes drag icon setup
+        """
+        context.set_icon_pixbuf(pixbuf, 0, 0)
+
+    def on_drag_motion(self, treeview, context, x, y, timestamp):
         """
             Called when a row is dragged over this treeview
         """
         if not self.receive:
-            return
+            return False
         self.enable_model_drag_dest(self.targets,
             gtk.gdk.ACTION_DEFAULT)
+        if self.drop_pos is None:
+            return False
         info = treeview.get_dest_row_at_pos(x, y)
-        if not info: return
-        treeview.set_drag_dest_row(info[0], info[1])
+        if not info:
+            return False
+        path, pos = info
+        if self.drop_pos == 'into':
+            # Only allow dropping into entries.
+            if pos == gtk.TREE_VIEW_DROP_BEFORE:
+                pos = gtk.TREE_VIEW_DROP_INTO_OR_BEFORE
+            elif pos == gtk.TREE_VIEW_DROP_AFTER:
+                pos = gtk.TREE_VIEW_DROP_INTO_OR_AFTER
+        elif self.drop_pos == 'between':
+            # Only allow dropping between entries.
+            if pos == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE:
+                pos = gtk.TREE_VIEW_DROP_BEFORE
+            elif pos == gtk.TREE_VIEW_DROP_INTO_OR_AFTER:
+                pos = gtk.TREE_VIEW_DROP_AFTER
+        treeview.set_drag_dest_row(path, pos)
+        context.drag_status(context.suggested_action, timestamp)
+        return True
 
-    def button_press(self, button, event):
+    def on_button_press(self, button, event):
         """
             The popup menu that is displayed when you right click in the
             playlist
@@ -256,7 +410,7 @@ class DragTreeView(gtk.TreeView):
         if path:
             if event.button != 3:
                 if event.type == gtk.gdk._2BUTTON_PRESS:
-                    self.cont.button_press(button, event)
+                    self.container.button_press(button, event)
 
                 if selection.count_selected_rows() <= 1:
                     return False
@@ -271,7 +425,7 @@ class DragTreeView(gtk.TreeView):
 
             if not selection.count_selected_rows():
                 selection.select_path(path[0])
-        return self.cont.button_press(button, event)
+        return self.container.button_press(button, event)
 
     #TODO maybe move this somewhere else? (along with _handle_unknown_drag_data)
     def get_drag_data(self, locs, compile_tracks = True, existing_tracks = []):
@@ -372,9 +526,9 @@ class SearchEntry(object):
             Called when the entry changes
         """
         if self.change_id:
-            gobject.source_remove(self.change_id)
+            glib.source_remove(self.change_id)
 
-        self.change_id = gobject.timeout_add(self.timeout,
+        self.change_id = glib.timeout_add(self.timeout,
             self.entry_activate)
 
     def on_entry_icon_press(self, entry, icon_pos, event):
@@ -396,41 +550,36 @@ class SearchEntry(object):
         """
         return getattr(self.entry, attr)
 
-def get_icon(id, size=gtk.ICON_SIZE_BUTTON):
+class VolumeControl(gtk.Alignment):
     """
-        Returns a stock icon for the specified id and size
+        Encapsulates a button and a slider to
+        control the volume indicating the current
+        status via icon and tooltip
     """
-    theme = gtk.icon_theme_get_default()
-    try:
-        icon = theme.load_icon(id, size, gtk.ICON_LOOKUP_NO_SVG)
-        if icon: return icon
-    except gobject.GError:
-        pass
+    def __init__(self):
+        gtk.Alignment.__init__(self, xalign=1)
 
-    # If no stock icon exists for the specified ID, search in the "images" data
-    # directory.
-    path = xdg.get_data_path('images', id + '.png')
-
-    # Fallback to the "track.png" file.
-    if not path:
-        path = xdg.get_data_path('images', 'track.png')
-
-    return gtk.gdk.pixbuf_new_from_file(path)
-
-class MuteButton(object):
-    """
-        Allows for immediate muting of the volume and
-        indicates the current volume level via an icon
-    """
-    def __init__(self, button):
-        self.button = button
         self.restore_volume = settings.get_option('player/volume', 1)
         self.icon_names = ['low', 'medium', 'high']
 
-        self.button.connect('toggled', self.on_toggled)
-        event.add_callback(self.on_setting_change, 'player_option_set')
+        builder = gtk.Builder()
+        builder.add_from_file(xdg.get_data_path('ui', 'widgets',
+            'volume_control.ui'))
+        builder.connect_signals(self)
 
-    def update_volume_icon(self, volume):
+        box = builder.get_object('volume_control')
+        box.reparent(self)
+
+        self.button = builder.get_object('button')
+        self.button.add_events(gtk.gdk.KEY_PRESS_MASK)
+        self.button_image = builder.get_object('button_image')
+        self.slider = builder.get_object('slider')
+        self.slider_adjustment = builder.get_object('slider_adjustment')
+        self.__update(self.restore_volume)
+
+        event.add_callback(self.on_option_set, 'player_option_set')
+
+    def __update(self, volume):
         """
             Sets the volume level indicator
         """
@@ -441,15 +590,43 @@ class MuteButton(object):
             i = int(round(volume * 2))
             icon_name = 'audio-volume-%s' % self.icon_names[i]
             #TRANSLATORS: Volume percentage
-            tooltip = '%d%%' % (volume * 100)
+            tooltip = _('%d%%') % (volume * 100)
 
         if volume == 1.0:
             tooltip = _('Full Volume')
 
-        self.button.child.set_from_icon_name(icon_name, gtk.ICON_SIZE_BUTTON)
-        self.button.set_tooltip_text(tooltip)
+        if volume > 0:
+            self.button.set_active(False)
 
-    def on_toggled(self, button):
+        self.button_image.set_from_icon_name(icon_name, gtk.ICON_SIZE_BUTTON)
+        self.button.set_tooltip_text(tooltip)
+        self.slider.set_value(volume)
+        self.slider.set_tooltip_text(tooltip)
+
+    def on_scroll_event(self, widget, event):
+        """
+            Changes the volume on scrolling
+        """
+        page_increment = self.slider_adjustment.page_increment
+        step_increment = self.slider_adjustment.step_increment
+        value = self.slider.get_value()
+
+        if event.direction == gtk.gdk.SCROLL_DOWN:
+            if event.state & gtk.gdk.SHIFT_MASK:
+                self.slider.set_value(value - page_increment)
+            else:
+                self.slider.set_value(value - step_increment)
+            return True
+        elif event.direction == gtk.gdk.SCROLL_UP:
+            if event.state & gtk.gdk.SHIFT_MASK:
+                self.slider.set_value(value + page_increment)
+            else:
+                self.slider.set_value(value + step_increment)
+            return True
+
+        return False
+
+    def on_button_toggled(self, button):
         """
             Mutes or unmutes the volume
         """
@@ -459,69 +636,45 @@ class MuteButton(object):
         else:
             volume = self.restore_volume
 
-        self.update_volume_icon(volume)
-
         if self.restore_volume > 0:
             settings.set_option('player/volume', volume)
 
-    def on_setting_change(self, event, sender, option):
+    def on_slider_value_changed(self, slider):
         """
-            Saves the restore volume and
-            changes the volume indicator
+            Stores the preferred volume
+        """
+        settings.set_option('player/volume', slider.get_value())
+
+    def on_slider_key_press_event(self, slider, event):
+        """
+            Changes the volume on key press
+            while the slider is focussed
+        """
+        page_increment = slider.get_adjustment().page_increment
+        step_increment = slider.get_adjustment().step_increment
+        value = slider.get_value()
+
+        if event.keyval == gtk.keysyms.Down:
+            slider.set_value(value - step_increment)
+            return True
+        elif event.keyval == gtk.keysyms.Page_Down:
+            slider.set_value(value - page_increment)
+            return True
+        elif event.keyval == gtk.keysyms.Up:
+            slider.set_value(value + step_increment)
+            return True
+        elif event.keyval == gtk.keysyms.Page_Up:
+            slider.set_value(value + page_increment)
+            return True
+
+        return False
+
+    def on_option_set(self, event, sender, option):
+        """
+            Updates the volume indication
         """
         if option == 'player/volume':
-            volume = settings.get_option(option, 1)
-
-            if volume > 0:
-                self.button.set_active(False)
-
-            self.update_volume_icon(volume)
-
-BITMAP_CACHE = dict()
-def get_text_icon(widget, text, width, height, bgcolor='#456eac',
-    bordercolor=None):
-    """
-        Gets a bitmap icon with the specified text, width, and height
-    """
-    if BITMAP_CACHE.has_key("%s - %sx%s - %s" % (text, width, height, bgcolor)):
-        return BITMAP_CACHE["%s - %sx%s - %s" % (text, width, height, bgcolor)]
-
-    default_visual = gtk.gdk.visual_get_system()
-    pixmap = gtk.gdk.Pixmap(None, width, height, default_visual.depth)
-    colormap = gtk.gdk.colormap_get_system()
-    white = colormap.alloc_color(65535, 65535, 65535)
-    black = colormap.alloc_color(0, 0, 0)
-    pixmap.set_colormap(colormap)
-    gc = pixmap.new_gc(foreground=black, background=white)
-
-    if not bordercolor: bordercolor = black
-    else:
-        bordercolor = colormap.alloc_color(gtk.gdk.color_parse(bordercolor))
-    gc.set_foreground(bordercolor)
-
-    pixmap.draw_rectangle(gc, True, 0, 0, width, height)
-    fg = colormap.alloc_color(gtk.gdk.color_parse(bgcolor))
-    gc.set_foreground(fg)
-    pixmap.draw_rectangle(gc, True, 1, 1, width - 2, height - 2)
-
-    layout = widget.create_pango_layout(str(text))
-    desc = pango.FontDescription("Sans 8")
-    layout.set_font_description(desc)
-    layout.set_alignment(pango.ALIGN_RIGHT)
-
-    gc.set_foreground(white)
-    inkRect, logicalRect = layout.get_pixel_extents()
-    l, b, w, h = logicalRect
-    x = ((width) / 2 - w / 2)
-    y = ((height) / 2 - h / 2)
-    pixmap.draw_layout(gc, x, y, layout)
-
-    pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, False, 8, width, height)
-    pixbuf = pixbuf.get_from_drawable(pixmap, colormap, 0, 0, 0,
-        0, width, height)
-
-    BITMAP_CACHE["%s - %sx%s - %s" % (text, width, height, bgcolor)] = pixbuf
-    return pixbuf
+            self.__update(settings.get_option(option, 1))
 
 class Menu(gtk.Menu):
     """
@@ -695,6 +848,7 @@ class Statusbar(object):
         frame.remove(label)
         hbox.pack_start(label, True, True)
         frame.add(hbox)
+
         for widget in children[1:]:
             # Bug in old PyGTK versions: Statusbar.remove hides
             # Container.remove.
@@ -707,6 +861,9 @@ class Statusbar(object):
         self.context_id = self.status_bar.get_context_id('status')
         self.message_ids = []
 
+        self.status_bar.set_app_paintable(True)
+        self.status_bar.connect('expose-event', self.on_expose_event)
+
     def set_status(self, status, timeout=0):
         """
             Sets the status message
@@ -714,7 +871,7 @@ class Statusbar(object):
         self.message_ids += [self.status_bar.push(self.context_id, status)]
 
         if timeout > 0:
-            gobject.timeout_add_seconds(timeout, self.clear_status)
+            glib.timeout_add_seconds(timeout, self.clear_status)
 
     def clear_status(self):
         """
@@ -754,121 +911,86 @@ class Statusbar(object):
             self.queue_count_label.hide_all()
             self.queue_count_label.set_no_show_all(True)
 
-class MenuRatingWidget(gtk.MenuItem):
-    """
-        A rating widget that can be used as a menu entry
-
-        @param: _get_tracks is a function that should return a list of tracks
-           linked to this widget.
-
-        @param: _get_tracks_rating is a function that should return an int
-           representing the rating of the tracks which are meant to be linked to
-           this widget. Should return 0 if two tracks have a different rating or
-           if it contains over miscellaneous/rating_widget_tracks_limit tracks.
-           Default limit: 100
-    """
-
-    def __init__(self, _get_tracks, _get_tracks_rating):
-        gtk.MenuItem.__init__(self)
-
-        self._get_tracks = _get_tracks
-        self._get_tracks_rating = _get_tracks_rating
-        self._last_calculated_rating = self._get_tracks_rating()
-
-        self.hbox = gtk.HBox(spacing=3)
-        self.hbox.pack_start(gtk.Label(_("Rating:")), False, False)
-        self.image = gtk.image_new_from_pixbuf(
-            self._get_rating_pixbuf(self._last_calculated_rating))
-        self.hbox.pack_start(self.image, False, False, 12)
-
-        self.add(self.hbox)
-
-        self.connect('button-release-event', self._update_rating)
-        self.connect('motion-notify-event', self._motion_notify)
-        self.connect('leave-notify-event', self._leave_notify)
-
-        # This may be useful when rating is changed from outside of the GUI
-        event.add_callback(self.on_rating_change, 'rating_changed')
-
-    def _motion_notify(self, widget, e):
-        steps = settings.get_option('miscellaneous/rating_steps', 5)
-        (x, y) = e.get_coords()
-        try:
-            (u, v) =  self.translate_coordinates(self.image, int(x), int(y))
-        except ValueError: return
-
-        if -12 <= u < 0:
-            r = 0
-        elif 0 <= u < steps*12:
-            r = (u / 12) + 1
+    def get_grip_edge(self, widget):
+        """
+            Taken from GTK source, retrieves the
+            preferred edge for the resize grip
+        """
+        if widget.get_direction() == gtk.TEXT_DIR_LTR:
+            edge = gtk.gdk.WINDOW_EDGE_SOUTH_EAST
         else:
-            r = -1
+            edge = gtk.gdk.WINDOW_EDGE_SOUTH_WEST
+        return edge
 
-        if r >= 0:
-            self.image.set_from_pixbuf(
-                self._get_rating_pixbuf(r))
-            self.queue_draw()
-
-    def _leave_notify(self, widget, e):
+    def get_grip_rect(self, widget):
         """
-            Resets the rating if the widget
-            is left without clicking
+            Taken from GTK source, retrieves the
+            rectangle to draw the resize grip on
         """
-        self.image.set_from_pixbuf(
-            self._get_rating_pixbuf(self._last_calculated_rating))
-        self.queue_draw()
+        width = height = 18
+        allocation = widget.get_allocation()
 
-    def _update_rating(self, widget, e):
+        width = min(width, allocation.width)
+        height = min(height, allocation.height - widget.style.ythickness)
+
+        if widget.get_direction() == gtk.TEXT_DIR_LTR:
+            x = allocation.x + allocation.width - width
+        else:
+            x = allocation.x + widget.style.xthickness
+
+        y = allocation.y + allocation.height - height
+
+        return gtk.gdk.Rectangle(x, y, width, height)
+
+    def on_expose_event(self, widget, event):
         """
-            Updates the rating of the tracks for this
-            widget, meant to be used with a click event
+            Override required to make alpha
+            transparency work properly
         """
-        event.remove_callback(self.on_rating_change, 'rating_changed')
+        if widget.get_has_resize_grip():
+            edge = self.get_grip_edge(widget)
+            rect = self.get_grip_rect(widget)
 
-        trs = self._get_tracks()
-        if trs and trs[0]:
-            steps = settings.get_option('miscellaneous/rating_steps', 5)
-            (x, y) = e.get_coords()
-            (u, v) =  self.translate_coordinates(self.image, int(x), int(y))
-            if -12 <= u < 0:
-                r = 0
-            elif 0 <= u < steps*12:
-                r = (u / 12) + 1
-            else:
-                r = -1
+            widget.style.paint_resize_grip(
+                widget.window,
+                widget.get_state(),
+                event.area,
+                widget,
+                'statusbar',
+                edge,
+                rect.x, rect.y,
+                rect.width - widget.style.xthickness,
+                rect.height - widget.style.ythickness
+            )
 
-            if r >= 0:
-                if r == self._last_calculated_rating:
-                    r = 0
+            frame = widget.get_children()[0]
+            box = frame.get_children()[0]
+            box.send_expose(event) # Bypass frame
 
-                for track in trs:
-                    track.set_rating(r)
+        return True
 
-                event.log_event('rating_changed', self, r)
-                self._last_calculated_rating = r
-
-        event.add_callback(self.on_rating_change, 'rating_changed')
-
-    def _get_rating_pixbuf(self, num):
-        """
-            Returns the pixbuf for num
-        """
-        return rating.rating_images[num]
-
-    def on_rating_change(self, type = None, object = None, data = None):
-        """
-            Handles possible changes of track ratings
-        """
-        self._last_calculated_rating = self._get_tracks_rating()
-        self.image.set_from_pixbuf(
-            self._get_rating_pixbuf(self._last_calculated_rating))
-        self.queue_draw ()
-
-def get_urls_for(items):
+class ProgressBarFormatter(formatter.ProgressTextFormatter):
     """
-        Returns the items' URLs
+        A formatter for progress bars
     """
-    return [item.get_loc_for_io() for item in items]
+    def __init__(self):
+        formatter.ProgressTextFormatter.__init__(self, self.get_option_value())
+
+        event.add_callback(self.on_option_set, 'gui_option_set')
+
+    def get_option_value(self):
+        """
+            Returns the current option value
+        """
+        return settings.get_option('gui/progress_bar_text_format',
+            '$current_time / $remaining_time')
+
+    def on_option_set(self, event, settings, option):
+        """
+            Updates the internal format on setting change
+        """
+        if option == 'gui/progress_bar_text_format':
+            self.props.format = self.get_option_value()
 
 def finish(repeat=True):
     """
@@ -878,17 +1000,4 @@ def finish(repeat=True):
         gtk.main_iteration()
         if not repeat: break
 
-def on_slider_key_press(widget, ev):
-    """
-        Called when the user presses a key when the volume bar is focused
-    """
-    # Modify default HScale up/down behaviour.
-    incr = widget.get_adjustment().page_size
-    if ev.keyval in (gtk.keysyms.Down, gtk.keysyms.Page_Down):
-        widget.set_value(widget.get_value() - incr)
-        return True
-    elif ev.keyval in (gtk.keysyms.Up, gtk.keysyms.Page_Up):
-        widget.set_value(widget.get_value() + incr)
-        return True
-    return False
-
+# vim: et sts=4 sw=4
