@@ -1,12 +1,12 @@
 from xl import main as ex, trax, common, event, xdg, settings, providers
-from xl.trax import search
-from xl.trax.util import sort_tracks
+from xl.trax import search, util
 from xl.nls import gettext as _
 from xlgui import panel, guiutil, playlist, menu
 import HTMLParser
 from StringIO import StringIO
 import Image
 import base64
+import glib
 import gobject
 import gtk
 import os
@@ -38,7 +38,7 @@ class ContextPopup(menu.TrackSelectMenu):
             Appends the selected tracks to the current playlist
         """
         selected = self.widget.get_selected_tracks()
-        pl = xlgui.controller().main.get_selected_playlist()
+        pl = xlgui.get_controller().main.get_selected_playlist()
         if pl:
             pl.playlist.add_tracks(selected)
 
@@ -47,7 +47,7 @@ class ContextPopup(menu.TrackSelectMenu):
             Called when the user clicks the "toggle queue" item
         """
         selected = self.widget.get_selected_tracks()
-        pl = xlgui.controller().main.get_selected_playlist()
+        pl = xlgui.get_controller().main.get_selected_playlist()
         ex.exaile().queue.add_tracks(selected)
         if pl:
             pl.playlist.add_tracks(selected)
@@ -80,7 +80,7 @@ class BrowserPage(webkit.WebView, providers.ProviderHandler):
         self.drag_source_set(
                     gtk.gdk.BUTTON1_MASK, self.targets,
                     gtk.gdk.ACTION_COPY|gtk.gdk.ACTION_MOVE)
-        self.drag_source_set_icon_stock('gtk-dnd')
+        self.drag_source_set_icon_stock(gtk.STOCK_DND)
 
         event.add_callback(self.on_playback_start, 'playback_track_start',
             ex.exaile().player)
@@ -98,7 +98,7 @@ class BrowserPage(webkit.WebView, providers.ProviderHandler):
         except:
             pass
 
-        gobject.idle_add(self.on_home_clicked)
+        glib.idle_add(self.on_home_clicked)
 
         # javascript debugger
         inspector = Inspector(self.get_web_inspector())
@@ -172,7 +172,7 @@ class BrowserPage(webkit.WebView, providers.ProviderHandler):
 
     def on_page_loaded(self, type=None, obj=None, data=None):
         self.refresh_button.set_sensitive(True)
-        self.refresh_button_image.set_from_stock('gtk-refresh', 1)
+        self.refresh_button_image.set_from_stock(gtk.STOCK_REFRESH, 1)
 
     def on_field_refresh(self, type=None, obj=None, data=None):
         self.execute_script(self.refresh_script % (data[0], u'%s' % data[1].replace('"', '\\"').replace('\n', '\\\n'), data[0]))
@@ -243,15 +243,15 @@ class BrowserPage(webkit.WebView, providers.ProviderHandler):
 
     def drag_begin(self, w, context):
         if self.hover == None:
-            self.drag_source_set_icon_stock("gtk-cancel")
+            self.drag_source_set_icon_stock(gtk.STOCK_CANCEL)
             return True
         self.dragging = True
 
         context.drag_abort(gtk.get_current_event_time())
         selection = self.get_selected_tracks()
         if len(selection)>1:
-            self.drag_source_set_icon_stock('gtk-dnd-multiple')
-        elif len(selection)>0: self.drag_source_set_icon_stock('gtk-dnd')
+            self.drag_source_set_icon_stock(gtk.STOCK_DND_MULTIPLE)
+        elif len(selection)>0: self.drag_source_set_icon_stock(gtk.STOCK_DND)
         return False
 
     def get_selected_tracks(self):
@@ -276,7 +276,7 @@ class BrowserPage(webkit.WebView, providers.ProviderHandler):
         tracks = self.get_selected_tracks()
         for track in tracks:
             guiutil.DragTreeView.dragged_data[track.get_loc_for_io()] = track
-        urls = guiutil.get_urls_for(tracks)
+        urls = util.get_uris_from_tracks(tracks)
         selection.set_uris(urls)
 
     def connect_events(self):
@@ -304,7 +304,7 @@ class BrowserPage(webkit.WebView, providers.ProviderHandler):
             Appends the selected tracks to the current playlist
         """
         selected = self.get_selected_tracks()
-        pl = xlgui.controller().main.get_selected_playlist()
+        pl = xlgui.get_controller().main.get_selected_playlist()
         if pl:
             pl.playlist.add_tracks(selected)
 
@@ -449,27 +449,29 @@ def get_track_tag(track, tag, default):
 def get_top_tracks(field, limit):
     tracks = [x.track for x in search.search_tracks_from_string(
                 ex.exaile().collection, '! %s==__null__' % field)]
-    tracks = sort_tracks([field], tracks, True)
+    tracks = util.sort_tracks([field], tracks, True)
     return tracks[:limit]
 
 def get_top_albums(field, limit):
     albums = [x.track for x in search.search_tracks_from_string(
                 ex.exaile().collection, '! %s==__null__' % field)]
-    albums = sort_tracks([field], albums, True)
+    albums = util.sort_tracks([field], albums, True)
     return albums[:limit]
 
 def get_top_artists(field, limit):
     artists = [x.artist for x in search.search_tracks_from_string(
                 ex.exaile().collection, '! %s==__null__' % field)]
-    artists = sort_tracks([field], artists, True)
+    artists = util.sort_tracks([field], artists, True)
     return artists[:limit]
 
 class ContextPage(object):
 
     TRACK_ICO_PATH = xdg.get_data_path('images/track.png')
     ARTIST_ICO_PATH = xdg.get_data_path("images/artist.png")
-    SEARCH_ICO_PATH = gtk.icon_theme_get_default().lookup_icon('gtk-find', gtk.ICON_SIZE_SMALL_TOOLBAR, gtk.ICON_LOOKUP_NO_SVG).get_filename()
-    ALBUM_ICO_PATH = gtk.icon_theme_get_default().lookup_icon('gtk-cdrom', gtk.ICON_SIZE_SMALL_TOOLBAR, gtk.ICON_LOOKUP_NO_SVG).get_filename()
+    SEARCH_ICO_PATH = gtk.icon_theme_get_default().lookup_icon(gtk.STOCK_FIND,
+        gtk.ICON_SIZE_SMALL_TOOLBAR, gtk.ICON_LOOKUP_NO_SVG).get_filename()
+    ALBUM_ICO_PATH = gtk.icon_theme_get_default().lookup_icon(gtk.STOCK_CDROM,
+        gtk.ICON_SIZE_SMALL_TOOLBAR, gtk.ICON_LOOKUP_NO_SVG).get_filename()
 
     def __init__(self, theme, base, template, async=[]):
         templatefile = open(theme.path+template)
@@ -592,11 +594,11 @@ class ContextPage(object):
     def get_rating_html(self, rating):
         html=''
         star = xdg.get_data_path("images", "star.png")
-        bstar = xdg.get_data_path("images", "brightstar.png")
-        steps = settings.get_option("miscellaneous/rating_steps", 5)
+        bstar = xdg.get_data_path("images", "emptystar.png")
+        maximum = settings.get_option("rating/maximum", 5)
         for i in range(rating):
             html+='<a href="rate://%d"><img src="file://%s"/></a>' % (i+1, star)
-        for i in range(steps-rating):
+        for i in range(maximum-rating):
             html+='<a href="rate://%d"><img src="file://%s"/></a>' % (rating+i+1, bstar)
         return html
 
@@ -1179,19 +1181,19 @@ class ContextPanel(gobject.GObject):
 
         return (self._child, self.name)
         
-def get_prefs_pane():
+def get_preferences_pane():
     return contextprefs
 
 def exaile_ready(object=None, a=None, b=None):
     global PANEL
-    PANEL = ContextPanel(xlgui.controller().main)
-    xlgui.controller().add_panel(*PANEL.get_panel())
+    PANEL = ContextPanel(xlgui.get_controller().main)
+    xlgui.get_controller().add_panel(*PANEL.get_panel())
 
 def enable(exaile):
     """
         Adds 'Contextual' tab to side panel
     """
-    if xlgui.controller():
+    if xlgui.get_controller():
         exaile_ready()
     else:
         event.add_callback(exaile_ready, 'gui_loaded')
@@ -1203,4 +1205,4 @@ def disable(exaile):
     """
     global PANEL
     event.remove_callback(exaile_ready, 'gui_loaded')
-    xlgui.controller().remove_panel(PANEL.get_panel()[0])
+    xlgui.get_controller().remove_panel(PANEL.get_panel()[0])
