@@ -19,14 +19,14 @@ import gtk
 from xl import event
 from xl.nls import gettext as _
 from xl import settings
-from xlgui import commondialogs
+from xlgui.widgets import dialogs
 import srprefs
 
 logger = logging.getLogger(__name__)
 
 STREAMRIPPER = None
 
-def get_prefs_pane():
+def get_preferences_pane():
     return srprefs
 
 class Streamripper(object):
@@ -61,7 +61,7 @@ class Streamripper(object):
                         subprocess.PIPE, subprocess.PIPE)
         except OSError:
             logger.error('There was an error executing streamripper')
-            commondialogs.error(self.exaile.gui.main.window, _('Error '
+            dialogs.error(self.exaile.gui.main.window, _('Error '
                     'executing streamripper'))
             return True
 
@@ -107,7 +107,7 @@ class Button(Streamripper):
         self.exaile = exaile
         self.button = gtk.ToggleButton()
         image = gtk.Image()
-        image.set_from_stock('gtk-media-record', gtk.ICON_SIZE_SMALL_TOOLBAR)
+        image.set_from_stock(gtk.STOCK_MEDIA_RECORD, gtk.ICON_SIZE_SMALL_TOOLBAR)
         self.button.set_image(image)
 
         toolbar = self.exaile.gui.play_toolbar
